"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const graphql_1 = require("graphql");
const postgraphile_1 = require("postgraphile");
exports.MutationAtomicityHeaderName = 'x-mutation-atomicity';
var MutationAtomicityHeaderValue;
(function (MutationAtomicityHeaderValue) {
    MutationAtomicityHeaderValue["ON"] = "on";
    MutationAtomicityHeaderValue["OFF"] = "off";
})(MutationAtomicityHeaderValue = exports.MutationAtomicityHeaderValue || (exports.MutationAtomicityHeaderValue = {}));
exports.getMutationAtomicityContext = (req) => {
    if (req.headers[exports.MutationAtomicityHeaderName] === 'on') {
        return req.mutationAtomicityContext;
    }
    else {
        return undefined;
    }
};
exports.AtomicMutationsHook = {
    ['postgraphile:httpParamsList'](paramsList, { req }) {
        if (paramsList.length > 1) {
            throw Error('AtomicMutationsPlugin does not support GraphQL query batching');
        }
        if (paramsList.length === 1) {
            const { query, operationName } = paramsList[0];
            const parsedQuery = graphql_1.parse(query);
            const [executedDefinition] = parsedQuery.definitions.filter((definition) => definition.name.value === operationName);
            if (executedDefinition &&
                executedDefinition.operation === 'mutation' &&
                req.headers[exports.MutationAtomicityHeaderName].toLowerCase() ===
                    MutationAtomicityHeaderValue.ON) {
                const totalMutations = executedDefinition.selectionSet.selections.filter((selection) => selection.name.value !== '__typename').length;
                req.mutationAtomicityContext = {
                    totalMutations,
                    executedMutations: [],
                };
            }
            else {
                req.headers[exports.MutationAtomicityHeaderName] =
                    MutationAtomicityHeaderValue.OFF;
            }
        }
        return paramsList;
    },
};
exports.AtomicMutationsPlugin = postgraphile_1.makeWrapResolversPlugin((context) => {
    if (context.scope.isRootMutation) {
        return { scope: context.scope };
    }
    else {
        return null;
    }
}, () => async (resolver, source, args, context, resolveInfo) => {
    const { mutationAtomicityContext, pgClient, } = context;
    try {
        const result = await resolver(source, args, context, resolveInfo);
        if (mutationAtomicityContext) {
            mutationAtomicityContext.executedMutations.push({ hasErrors: false });
        }
        return result;
    }
    catch (error) {
        if (mutationAtomicityContext) {
            mutationAtomicityContext.executedMutations.push({ hasErrors: true });
        }
        throw error;
    }
    finally {
        if (mutationAtomicityContext &&
            mutationAtomicityContext.executedMutations.length ===
                mutationAtomicityContext.totalMutations &&
            mutationAtomicityContext.executedMutations.filter((mutation) => mutation.hasErrors === true).length) {
            pgClient.query(`rollback`);
        }
    }
});
